/*
 * ControlLight_Lampa.c
 *
 * Created: 23. 9. 2021 18:25:46
 * Author : jaro 

Ovldanie LED - Lampa
- tlaidlo 1 - kratke staenie - zapn / vypn; dlh prida / ubra intenzitu

Procesor ATtiny 45 - 1MHz internal

Fuse bits setup:    X - mark changes

High byte Extended:
SELFPRGEN - Self Programming Enable (1 unprogrammed)

High byte:
RSTDISBL -  External Reset Disable (1 unprogrammed)
DWEN - debugWIRE Enable (1 unprogrammed)
SPIEN - Enable Serial Program and Data Downloading (0 programmed)
WDTON - Watchdog Timer Always On (1 unprogrammed)
EESAVE - EEPROM memory is preserved through the Chip Erase (0 programmed)
BODLEVEL2 - Brown-out Detector trigger level (1 unprogrammed) - (BOD disable)
BODLEVEL1 - Brown-out Detector trigger level (1 unprogrammed)
BODLEVEL0 - Brown-out Detector trigger level (1 unprogrammed)

Low Byte:
CKDIV8 - Divide clock by 8 (0 unprogrammed)
CKOUT - Clock output enabled 1 (unprogrammed)
SUT1 - Select start-up time (1 unprogrammed)
SUT0 - Select start-up time (0 programmed)
CKSEL3 - Select Clock source (0 programmed)
CKSEL2 - Select Clock source (0 programmed)
CKSEL1 - Select Clock source (1 unprogrammed) (internal oscilator 8MHz / 8 = 1MHz)
CKSEL0 - Select Clock source (0 programmed)

*/

#include <avr/io.h>
#include <avr/interrupt.h>
#define F_CPU 1000000UL  //1 MHz
#include <avr/delay.h>
#include <avr/sleep.h>
#include <avr/eeprom.h>

//Defincia vstupov a vstupov
#define O_Light		PB0		//Vstup - riadenie led
#define O_LightTl	PB1		//Vstup - podsvietenuie tlaidla
#define	O_PB2		PB2		//Vstup - Zapnanie odporu pre meranie naptia batrie - SCKinf

#define	I_Tl1		PINB3	//Vstup - tlaidlo 1
#define I_ADC2		PINB4   //Vstup - analogov vstup - naptie batrie
#define I_PB5		PINB5   //Vstup - NC / reset

#define CheckBit(Adress,BIT) (Adress&(1<<BIT))          //pre kontrolu bitov
#define SetBit(Adress,BIT) (Adress|=(1<<BIT))           //pre nahodenie bitu
#define ClearBit(Adress,BIT) (Adress&=~(1<<BIT))         //pre zhodenie bit

#define Sleep_Mode 0
#define ControlLight_Mode 1
#define SetTime_Mode	2
#define  Off_Dim 1				// vypnutie so stmievanm
#define  Off_Timer 1			// asova vypnutia

#define PWM_Min 10		// definovaie minimlne PWM
#define PWM1 OCR0A		// definovaie registra pre PWM1
#define PWM2 OCR0B		// definovaie registra pre PWM1

#define	Bat_Set 33	// hodnota naptia batrie * 10 ; 3,3V *10 = 33
#define	Bat_Value Bat_Set*5

struct	//bitov premenn
{
unsigned char Tl1_Off :1;				// flag - tlaidlo 1 bolo vypnut
unsigned char PWM_On :1;				// flag - Svetlo je zapnut
unsigned char PWM_Plus :1;				// flag - Smer regulcie intenzity
unsigned char PWM_ToDark :1;			// flag - stmevanie
}F;		//bitov premenn

uint16_t Count_Tl1;			// potadlo pre tlaidlo 1
//uint16_t Count_Tl2;			// potadlo pre tlaidlo 2
uint16_t Count_ms;			// potadlo ms
uint16_t Count_Light;		// poitadlo pre LED svietenia
uint16_t Last_LightTime;	// pre meranie naptia batrie
uint16_t Light_Time=60;		// doba trvanie svietenie v sekundch
uint8_t Mode;				// slo mdu
//uint8_t Step_Light;			// krok svietenie LED
//uint8_t Count_Steps = 0;
uint8_t PWM_Value = 128;			// hodnota PWM - intenzita
uint8_t PWM2_Value = 255;			// hodnota PWM2 - intenzita

uint16_t EEMEM	E_LightTime;	//uloen hodnota trvania osvetlenia

//Nastavenie Fuse bitov sa pripe do sboru s prponou *.elf a obvod mono naprogramova priamo.
FUSES =
{
.low = (FUSE_CKDIV8 & FUSE_SUT0 & FUSE_CKSEL0 & FUSE_CKSEL2 & FUSE_CKSEL3),		//zapsa iba programovan bity - 0 alebo default hodnoty = LFUSE_DEFAULT (pre cel byte)
.high = (FUSE_SPIEN & FUSE_EESAVE),	//default = HFUSE_DEFAULT
};

// Prechod do Sleep modu - PowerDown
void Mode_ToSleep(void)
{
	if(Count_Tl1==0 && F.Tl1_Off==1)
	{
		ClearBit(ADCSRA,ADEN);	// zakzanie meranie naptia
		ClearBit(PORTB,O_PB2);	// vypnutie odporu pre meranie naptia
		PWM1=0x00;
		sleep_enable();
		ClearBit(PORTB,O_LightTl);
		_delay_ms(5);
		sleep_cpu();
		sleep_disable();

		_delay_ms(5);
		Count_Light=0;
		Count_ms=0;
		SetBit(ADCSRA,ADEN);	// povolenie meranie naptia
		SetBit(PORTB,O_PB2);	// zapnutie odporu pre meranie naptia
		Mode=ControlLight_Mode;
	}

}


// ovldanie intenzity
void Mode_ControlLight(void)
{
	if(F.Tl1_Off)
	{
		if(!F.PWM_ToDark) ClearBit(PORTB,O_LightTl);
		if(Count_Tl1<2000 && Count_Tl1>10)
		{
			if(F.PWM_On)
			{
				if(!F.PWM_ToDark)
				{
					if(Off_Dim==1) Count_Light=Light_Time+1;		// postupn vypnutie
					else
					{
						// okamit vypnutie
						F.PWM_On=0, PWM1=0;
						Mode=Sleep_Mode;
					}
				}
				else 
				{
					PWM1=PWM_Value, ClearBit(PORTB,O_LightTl), F.PWM_ToDark=0;
					Count_Light=0;
					Count_ms=0;
				}
			}
			else
			{
				F.PWM_On=1, PWM1=PWM_Value;
				Count_Light=0;
				Count_ms=0;
			}
		}
		else if(Count_Tl1>=2000)
		{
			if(F.PWM_Plus) F.PWM_Plus=0;
			else F.PWM_Plus=1;
			Count_Light=0;
			Count_ms=0;
		}
		Count_Tl1=0;
	} 
	else
	{
		Count_Light=0;
		Count_ms=0;
		if(F.PWM_On)
		{
			if(Count_Tl1>=2015)
			{
				if(F.PWM_Plus)
				{
					if(PWM_Value<255) ++PWM_Value;
					else SetBit(PORTB,O_LightTl);
				}
				else
				{
					if(--PWM_Value<PWM_Min) PWM_Value=PWM_Min, SetBit(PORTB,O_LightTl);
				}
				PWM1=PWM_Value;
				Count_Tl1=2000;
			}
		}
		else
		{
			if(Count_Tl1>5000)
			{
				Count_ms=0;
				Count_Light=0;
				Light_Time=0;
				PlusMinuta();
				Mode=SetTime_Mode;
				Count_Tl1=0;
			}
			
		}
	}
	
	// Kontrola doby svietenia	
	uint8_t x;
	
	if(Count_Light>Light_Time && F.PWM_On)
	{
		F.PWM_ToDark=1;
		SetBit(PORTB,O_LightTl);
		Count_Light=Light_Time;
		x=PWM_Value/10;
		if(PWM1>x)
		{
			PWM1=PWM1-x;	// zniovanie intenzity
			F.PWM_ToDark=1;
		}
		else
		{
			F.PWM_ToDark=0;
			Count_Light=0;	// vypnutie svetla a do Sleep modu
			Count_ms=0;
			F.PWM_On=0;
			PWM1=0;
			Mode=Sleep_Mode;
		}
	}
	
	// Meranie naptia batrie - nastavi modulo kadch 5 seknd?
	if(Count_Light % 5==0)
	{
		if(Last_LightTime!=Count_Light)	// aby sa neopakovalo meranie v tej istej sekunde
		{
			Last_LightTime=Count_Light;
			// Start the AD conversion
			ADCSRA|=(1<<ADSC);	
		}
	}
};

// Uloenie asu do eeprom
void SaveTime(void)
{
	eeprom_write_word(&E_LightTime, Light_Time);
	
	SetBit(PORTB,O_LightTl);	// bliknutie
	_delay_ms(100);
	ClearBit(PORTB,O_LightTl);
	_delay_ms(200);
	SetBit(PORTB,O_LightTl);	// bliknutie
	_delay_ms(100);
	ClearBit(PORTB,O_LightTl);
	while(!F.Tl1_Off);	// akaj na pustenie tlaidla
	Mode=Sleep_Mode;
	Count_Tl1=0;
	Count_ms=0;
	Count_Light=0;
}

// Plus 1 minta
void PlusMinuta(void)
{
	if(Light_Time<3600) Light_Time=Light_Time+60;
	SetBit(PORTB,O_LightTl);	// bliknutie
	_delay_ms(100);
	ClearBit(PORTB,O_LightTl);
	while(!F.Tl1_Off);	// akaj na pustenie tlaidla
	Count_Tl1=0;
	Count_ms=0;
	Count_Light=0;
}

// Nastavenie asu vypnania
void Mode_SetTime(void)
{
	if(F.Tl1_Off)
	{
		if(Count_Tl1>=10) PlusMinuta();
	}
	else
	{
		if(Count_Tl1>5000) SaveTime();
	}
	if(Count_Light>10) Mode=Sleep_Mode;		// ak nebolo stlaen tl 10 sek. - ods bez zpisu
}

// Pretanie analgovho vstupu - meranie naptia batrie
void Battery_Volt(void)
{
	// referencia 2,56V - nastaven odpormi 10k a 10k - (5V na batrii = 2,5V na ADC2 = 255) 
	
	// 3,2V = 160 ADC2
	if(ADCH<Bat_Value)	// naptie je menie ako 3,3V
	//if(ADCH<200)	// naptie je menie ako 4,2V
	{
		SetBit(PORTB,O_LightTl);	// bliknutie
		_delay_ms(100);
		ClearBit(PORTB,O_LightTl);
	}
	ADCSRA|=(1<<ADIF);	// reset bitu preruenia

}

// Timer 1 - 1kHz - 1ms
ISR(TIM1_COMPA_vect)
{
	if(CheckBit( PINB,I_Tl1)==0)
	{
		F.Tl1_Off=0;
		if(++Count_Tl1>10000)
		{
			Count_Tl1=10000;
		}
	}
	else
	{
		F.Tl1_Off=1;
	}

	if(++Count_ms>=1000)
	{
		Count_ms=0;
		if(++Count_Light>60000) Count_Light=60000;	
	}
}

ISR(PCINT0_vect)
{
//_delay_us(1);
if(CheckBit( PINB,I_Tl1)==0);
//if(CheckBit( PINB,I_Tl2)==0) F.Tl_On=1;

}

void Setup_Chip(void)
{
// Declare your local variables here

// Crystal Oscillator division factor: 8
CLKPR=(1<<CLKPCE);
CLKPR=(0<<CLKPCE) | (0<<CLKPS3) | (0<<CLKPS2) | (1<<CLKPS1) | (1<<CLKPS0);

// Input/Output Ports initialization
// Port B initialization
// Function: Bit5=In Bit4=In Bit3=In Bit2=Out Bit1=Out Bit0=Out
DDRB=(0<<DDB5) | (0<<DDB4) | (0<<DDB3) | (1<<DDB2) | (1<<DDB1) | (1<<DDB0);
// State: Bit5=T Bit4=T Bit3=P Bit2=0 Bit1=0 Bit0=0
PORTB=(0<<PB5) | (0<<PB4) | (1<<PB3) | (0<<PB2) | (0<<PB1) | (0<<PB0);

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: 125,000 kHz
// Mode: Phase correct PWM top=0xFF
// OC0A output: Non-Inverted PWM
// OC0B output: Disconnected
// Timer Period: 4,08 ms
// Output Pulse(s):
// OC0A Period: 4,08 ms Width: 0 us
TCCR0A=(1<<COM0A1) | (0<<COM0A0) | (0<<COM0B1) | (0<<COM0B0) | (0<<WGM01) | (1<<WGM00);
TCCR0B=(0<<WGM02) | (0<<CS02) | (0<<CS01) | (1<<CS00);
TCNT0=0x00;
OCR0A=0x00;
OCR0B=0x00;

// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: 15,625 kHz
// Mode: CTC top=OCR1C
// OC1A output: Disconnected
// OC1B output: Disconnected
// Timer Period: 8 ms
// Timer1 Overflow Interrupt: Off
// Compare A Match Interrupt: On
// Compare B Match Interrupt: Off
PLLCSR=(0<<PCKE) | (0<<PLLE) | (0<<PLOCK);

TCCR1=(1<<CTC1) | (0<<PWM1A) | (0<<COM1A1) | (0<<COM1A0) | (0<<CS13) | (1<<CS12) | (0<<CS11) | (0<<CS10);
GTCCR=(0<<TSM) | (0<<PWM1B) | (0<<COM1B1) | (0<<COM1B0) | (0<<PSR1) | (0<<PSR0);
TCNT1=0x00;
OCR1A=0x7C;
OCR1B=0x00;
OCR1C=0x7C;

// Timer(s)/Counter(s) Interrupt(s) initialization
TIMSK=(1<<OCIE1A) | (0<<OCIE1B) | (0<<OCIE0A) | (0<<OCIE0B) | (0<<TOIE1) | (0<<TOIE0);

// External Interrupt(s) initialization
// INT0: Off
// Interrupt on any change on pins PCINT0-5: On
GIMSK=(0<<INT0) | (1<<PCIE);
MCUCR=(0<<ISC01) | (0<<ISC00);
PCMSK=(0<<PCINT5) | (0<<PCINT4) | (1<<PCINT3) | (0<<PCINT2) | (0<<PCINT1) | (0<<PCINT0);
GIFR=(0<<INTF0) | (1<<PCIF);

// USI initialization
// Mode: Disabled
// Clock source: Register & Counter=no clk.
// USI Counter Overflow Interrupt: Off
USICR=(0<<USISIE) | (0<<USIOIE) | (0<<USIWM1) | (0<<USIWM0) | (0<<USICS1) | (0<<USICS0) | (0<<USICLK) | (0<<USITC);

// Analog Comparator initialization
// Analog Comparator: Off
// The Analog Comparator's positive input is
// connected to the AIN0 pin
// The Analog Comparator's negative input is
// connected to the AIN1 pin
ACSR=(1<<ACD) | (0<<ACBG) | (0<<ACO) | (0<<ACI) | (0<<ACIE) | (0<<ACIS1) | (0<<ACIS0);
ADCSRB=(0<<ACME);
// Digital input buffer on AIN0: On
// Digital input buffer on AIN1: On
DIDR0=(0<<AIN0D) | (0<<AIN1D);

/// ADC initialization
// ADC Clock frequency: 62,500 kHz
// ADC Voltage Reference: 2.56V, AREF discon.
// ADC Bipolar Input Mode: Off
// ADC Reverse Input Polarity: Off
// ADC Auto Trigger Source: ADC Stopped
// Only the 8 most significant bits of
// the AD conversion result are used
// Digital input buffers on ADC0: On, ADC1: On, ADC2: Off, ADC3: On
DIDR0|=(0<<ADC0D) | (1<<ADC2D) | (0<<ADC3D) | (0<<ADC1D);
ADMUX=(1<<REFS1) | (0<<REFS0) | (1<<REFS2) | (1<<ADLAR) | (0<<MUX3) | (0<<MUX2) | (1<<MUX1) | (0<<MUX0);
ADCSRA=(1<<ADEN) | (0<<ADSC) | (0<<ADATE) | (0<<ADIF) | (0<<ADIE) | (1<<ADPS2) | (0<<ADPS1) | (0<<ADPS0);
ADCSRB=(0<<BIN) | (0<<IPR) | (0<<ADTS2) | (0<<ADTS1) | (0<<ADTS0);

//Nastavenie PowerDown mdu
//MCUCR=(1<<SM1) | (0<<SM0);		//nastavenie Power Down mdu
set_sleep_mode(SLEEP_MODE_PWR_DOWN);
}

int main(void)
{
	Setup_Chip();
	
	sei();		// povolenie preruenia

	uint16_t TempE;
	TempE=eeprom_read_word(&E_LightTime);
	if(TempE==0xFFFF)
	{
		eeprom_write_word(&E_LightTime, Light_Time);		//vchodzia hodnota pre trvanie osvetlenia
	}
	else Light_Time=TempE;

	while (1)
	{
	//_delay_us(1);

		switch (Mode)
		{
			case 0: Mode_ToSleep(); break;
			case 1: Mode_ControlLight(); break;
			case 2: Mode_SetTime(); break;
		}
		
		// kontrola merania naptia batrie
		if(CheckBit(ADCSRA,ADIF)) Battery_Volt();
	}

}

